/*
    SPDX-FileCopyrightText: 2020 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

// list all headers here that have no .cpp file
// this only serves as a guarantee that these headers actually compile
#include "jniargument.h"
#include "jniarray.h"
#include "jnimethod.h"
#include "jniobject.h"
#include "jnipp.h"
#include "jniprimitivetypes.h"
#include "jniproperty.h"
#include "jnireturnvalue.h"
#include "jnisignature.h"
#include "jnitypes.h"
#include "jnitypetraits.h"
