/*
    SPDX-FileCopyrightText: 2020 Volker Krause <vkrause@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

// list all headers here that have no .cpp file
// this only serves as a guarantee that these headers actually compile
#include "androidtypes.h"
#include "calendarcontract.h"
#include "manifestpermission.h"
#include "openablecolumns.h"
#include "settings.h"
